// Copyright 2019 github.com. All rights reserved.
// Use of this source code is governed by github.com.

package vcode

import (
	"context"
	"fmt"
	"property-thirdparty/consts"
	"property-thirdparty/errors"
	"property-thirdparty/parser"
	pb_v1 "property-thirdparty/pb/v1"
	"property-thirdparty/thirdparty"
	"property-thirdparty/utils"
	"strconv"

	"git.getensh.com/common/gopkgs/cache"
	"git.getensh.com/common/gopkgs/logger"
	"go.uber.org/zap"
	"google.golang.org/grpc/status"
	json2 "encoding/json"
)

type JsonStruct struct {

}
func (p *JsonStruct)MarshalToString(v interface{}) (string, error) {
	bytes, err := json2.Marshal(v)
	return string(bytes), err
}
func (p *JsonStruct)Marshal(v interface{}) ([]byte, error) {
	bytes, err := json2.Marshal(v)
	return bytes, err
}
func (p *JsonStruct)Unmarshal(bytes []byte, v interface{}) (error) {
	err := json2.Unmarshal(bytes, v)
	return err
}
var json = &JsonStruct{}


// GetVcode 短信验证接口
func GetVcode(ctx context.Context, req *pb_v1.GetVcodeRequest) (reply *pb_v1.GetVcodeReply, err error) {
	reply = &pb_v1.GetVcodeReply{}

	// 捕获各个task中的异常并返回给调用者
	defer func() {
		if r := recover(); r != nil {
			err = fmt.Errorf("%+v", r)
			e := &status.Status{}
			if er := json.Unmarshal([]byte(err.Error()), e); er != nil {
				logger.Error("err",
					zap.String("system_err", err.Error()),
					zap.Stack("stacktrace"))
			}
		}
	}()

	switch {
	case len(req.PhoneNumber) != 11 || !utils.VerifyMobileFormat(req.PhoneNumber):
		return reply, status.Error(10003, "参数错误,手机号错误")
	}

	// 发送验证码
	code := utils.RankVCode(1000, 9999)
	data := []interface{}{code, strconv.Itoa(consts.SMS_VALIDITY) + "分钟"}

	// request
	if true {
		datas := []string{fmt.Sprintf("%d", code), strconv.Itoa(consts.SMS_VALIDITY) + "分钟"}
		err = thirdparty.TencentMsg(req.PhoneNumber, datas, parser.Conf.ThirdParty.TencentMsg.VcodeTemplateId)
		if err != nil {
			logger.Error("func",
				zap.String("call", "ThirdParty.TencentMsg"),
				zap.String("params", req.PhoneNumber),
				zap.String("error", err.Error()))
			return reply, errors.VCodeSendFailed
		}
	} else if err = thirdparty.SendTemplateSMS(req.PhoneNumber, data, parser.Conf.ThirdParty.RongLian.VcodeTemplateId); err != nil {
		logger.Error("func",
			zap.String("call", "ThirdParty.SendTemplateSMS"),
			zap.String("params", req.PhoneNumber),
			zap.String("error", err.Error()))
		return reply, errors.VCodeSendFailed
	}

	// 存入Redis
	redisKey := ""

	redisKey += consts.VCODEKEY + req.PhoneNumber

	// save to redis
	_, err = cache.Redis().SetEx(redisKey, consts.SMS_VALIDITY*60, code)

	if err != nil {
		logger.Error("redis",
			zap.String("call", "SetEx"),
			zap.String("params", redisKey),
			zap.String("error", err.Error()))
		return reply, errors.SystemError
	}

	reply.Vcode = uint32(code)

	return
}
