package wxapplet

import (
	"fmt"
	"git.getensh.com/common/gopkgs/cache"
	"github.com/tidwall/gjson"
	"io/ioutil"
	"net/http"
	"errors"
	"property-thirdparty/parser"
)

const AccessTokenUrl = "https://api.weixin.qq.com/cgi-bin/token"
const AccessTokenRedisKey = "wxapplet_access_token"
func GetAccessToken() (string, error) {
	token, _ := cache.Redis().Get(AccessTokenRedisKey)
	if token != "" {
		return token, nil
	}

	url := fmt.Sprintf("%s?grant_type=client_credential&appid=%s&secret=%s",
		parser.Conf.ThirdParty.Wx.AppletAppId, parser.Conf.ThirdParty.Wx.AppletAppSecret)
	request, err := http.NewRequest("GET", url, nil)
	if err != nil {
		return "", err
	}


	client := http.Client{}
	resp, err := client.Do(request)
	if err != nil {
		return "", err
	}
	defer resp.Body.Close()

	respBytes, err := ioutil.ReadAll(resp.Body)
	if err != nil {
		return "", err
	}

	token = gjson.GetBytes(respBytes, "access_token").String()
	expire := gjson.GetBytes(respBytes, "expires_in").Int()
	errmsg := gjson.GetBytes(respBytes, "errmsg").String()
	errcode := gjson.GetBytes(respBytes, "errcode").Int()
	if errcode == 0 {
		cache.Redis().SetEx(AccessTokenRedisKey, expire - 10, token)
		return token, nil
	}
	return "", errors.New(fmt.Sprintf("%d %s", errcode, errmsg))
}
