package wxapplet

import (
	"fmt"
	"github.com/go-pay/gopay"
	"github.com/go-pay/gopay/wechat"
	"go.uber.org/zap"
	"google.golang.org/grpc/status"
	"property-thirdparty/errors"
	"strconv"
	"time"
	"math/rand"
	"git.getensh.com/common/gopkgs/logger"
	"context"
	"property-thirdparty/parser"
	pb_v1 "property-thirdparty/pb/v1"
	wechatv3 "github.com/go-pay/gopay/wechat/v3"

)


func Krand(size int, kind int) []byte {
	ikind, kinds, result := kind, [][]int{[]int{10, 48}, []int{26, 97}, []int{26, 65}}, make([]byte, size)
	is_all := kind > 2 || kind < 0
	rand.Seed(time.Now().UnixNano())
	for i := 0; i < size; i++ {
		if is_all { // random ikind
			ikind = rand.Intn(3)
		}
		scope, base := kinds[ikind][0], kinds[ikind][1]
		result[i] = uint8(base + rand.Intn(scope))
	}
	return result
}

const (
	KC_RAND_KIND_NUM   = 0 // 纯数字
	KC_RAND_KIND_LOWER = 1 // 小写字母
	KC_RAND_KIND_UPPER = 2 // 大写字母
	KC_RAND_KIND_ALL   = 3 // 数字、大小写字母
)

type WxPrepay struct {
	AppID string `json:"appId"`
	//PartnerID string `json:"partner_id"`
	//PrepayID  string `json:"prepay_id"`
	Package   string `json:"package"`
	NonceStr  string `json:"nonceStr"`
	Timestamp int64  `json:"timeStamp"`
	SignType  string `json:"signType"`
	Sign      string `json:"paySign"`
}

type PrePayBaseInfo struct {
	AppId string
	MchId string
	ApiKey string
	OpenId string
	NotifyUrl string
	ProductName string
	ClientIp string
	Amount int64
	OrderNo string
	Native bool

}

func prePayV3(info *PrePayBaseInfo) (string, string, string, error) {
	if info.Native {
		return "", "", "", status.Error(10006, "暂不支持二维码付款")
	}
	mchID := parser.Conf.ThirdParty.Wx.MerchantMchID
	mchCertificateSerialNumber := parser.Conf.ThirdParty.Wx.MerchantMchCertificateSerialNumber
	mchAPIv3Key := parser.Conf.ThirdParty.Wx.MerchantMchAPIv3Key

	clientv3, err := wechatv3.NewClientV3(mchID, mchCertificateSerialNumber, mchAPIv3Key, parser.Conf.ThirdParty.Wx.MerchantMchPrivateKey)
	if err != nil {
		logger.Error("func",
			zap.String("call", "prePayV3"),
			zap.String("error", "newclientv3:"+err.Error()))
		return "", "", "",status.Error(10006, fmt.Sprintf("预支付失败"))
	}

	bm := make(gopay.BodyMap)
	bm.Set("sp_appid", parser.Conf.ThirdParty.Wx.AppletAppId)
	bm.Set("sp_mchid", mchID)
	//bm.Set("sub_mchid", "1628975670")
	bm.Set("sub_mchid", info.MchId)
	bm.Set("description", info.ProductName)
	bm.Set("out_trade_no", info.OrderNo)
	//bm.Set("notify_url", "https://www.wxmini.getensh.com/apis/v1/wx/applet/pay_callback")
	bm.Set("notify_url", info.NotifyUrl)

	subBm := make(gopay.BodyMap)
	//subBm.Set("total", 1)
	subBm.Set("total", info.Amount)
	bm.Set("amount", subBm)
	subBm = make(gopay.BodyMap)
	subBm.Set("sp_openid", info.OpenId)
	bm.Set("payer", subBm)
	rsp, err := clientv3.V3PartnerTransactionJsapi(bm)
	if err != nil {
		logger.Error("func",
			zap.String("call", "prePayV3"),
			zap.String("error", "V3PartnerTransactionJsapi:"+err.Error()))
		return "", "", "",status.Error(10006, fmt.Sprintf("预支付失败"))
	}
	if rsp.Response.PrepayId == "" {
		logger.Error("func",
			zap.String("call", "prePayV3"),
			zap.String("error", "V3PartnerTransactionJsapi:"+rsp.Error))
		return "", "", "",status.Error(10006, fmt.Sprintf("预支付失败"))
	}

	sign, err := clientv3.PaySignOfApplet(parser.Conf.ThirdParty.Wx.AppletAppId, rsp.Response.PrepayId)
	if err != nil {
		logger.Error("func",
			zap.String("call", "prePayV3"),
			zap.String("error", "PaySignOfApplet:"+err.Error()))
		return "", "", "",status.Error(10006, fmt.Sprintf("预支付失败"))
	}
	bytes, _ := json.Marshal(sign)
	return string(bytes), rsp.Response.PrepayId, "", nil
}

func prePayV3Native(info *PrePayBaseInfo) (string, string, string, error) {
	mchID := parser.Conf.ThirdParty.Wx.MerchantMchID
	mchCertificateSerialNumber := parser.Conf.ThirdParty.Wx.MerchantMchCertificateSerialNumber
	mchAPIv3Key := parser.Conf.ThirdParty.Wx.MerchantMchAPIv3Key

	clientv3, err := wechatv3.NewClientV3(mchID, mchCertificateSerialNumber, mchAPIv3Key, parser.Conf.ThirdParty.Wx.MerchantMchPrivateKey)
	if err != nil {
		logger.Error("func",
			zap.String("call", "prePayV3"),
			zap.String("error", "newclientv3:"+err.Error()))
		return "", "", "",status.Error(10006, fmt.Sprintf("预支付失败"))
	}

	bm := make(gopay.BodyMap)
	bm.Set("sp_appid", parser.Conf.ThirdParty.Wx.AppletAppId)
	bm.Set("sp_mchid", mchID)
	//bm.Set("sub_mchid", "1628975670")
	bm.Set("sub_mchid", info.MchId)
	bm.Set("description", info.ProductName)
	bm.Set("out_trade_no", info.OrderNo)
	//bm.Set("notify_url", "https://www.wxmini.getensh.com/apis/v1/wx/applet/pay_callbackv3")
	bm.Set("notify_url", info.NotifyUrl)

	subBm := make(gopay.BodyMap)
	//subBm.Set("total", 1)
	subBm.Set("total", info.Amount)
	bm.Set("amount", subBm)
	//subBm = make(gopay.BodyMap)
	//subBm.Set("sp_openid", info.OpenId)
	//bm.Set("payer", subBm)
	rsp, err := clientv3.V3PartnerTransactionNative(bm)
	if err != nil {
		logger.Error("func",
			zap.String("call", "prePayV3"),
			zap.String("error", "V3PartnerTransactionNative:"+err.Error()))
		return "", "", "",status.Error(10006, fmt.Sprintf("预支付失败"))
	}
	if rsp.Response.CodeUrl == "" {
		logger.Error("func",
			zap.String("call", "prePayV3"),
			zap.String("error", "V3PartnerTransactionNative:"+rsp.Error))
		return "", "", "",status.Error(10006, fmt.Sprintf("预支付失败"))
	}
	fmt.Printf("qcode url:%s\n", rsp.Response.CodeUrl)

	return "", "", rsp.Response.CodeUrl, nil
}

func prePay(info *PrePayBaseInfo) (string, string, string, error) {
	if true {
		return prePayV3(info)
	}

	client := wechat.NewClient(info.AppId, info.MchId, info.ApiKey, true)

	//设置国家
	client.SetCountry(wechat.China)

	//初始化参数Map
	bm := make(gopay.BodyMap)
	bm.Set("appid", info.AppId)
	bm.Set("body", info.ProductName)
	bm.Set("mch_id", info.MchId)
	bm.Set("notify_url", info.NotifyUrl)

	bm.Set("spbill_create_ip", info.ClientIp)
	bm.Set("total_fee", info.Amount)
	bm.Set("out_trade_no", info.OrderNo)
	bm.Set("nonce_str", string(Krand(32, KC_RAND_KIND_ALL)))

	bm.Set("sign_type", wechat.SignType_MD5)
	if info.Native {
		bm.Set("trade_type", wechat.TradeType_Native)
		bm.Set("product_id", info.OrderNo)
	} else {
		bm.Set("trade_type", wechat.TradeType_Mini)
		bm.Set("openid", info.OpenId)
	}

	resp, err := client.UnifiedOrder(bm)
	if err != nil {
		param := bm.JsonBody()
		logger.Error("func",
			zap.String("call", "WxPay_UnifiedOrder"),
			zap.String("params", param),
			zap.String("error", err.Error()))
		return "", "", "",status.Error(10006, fmt.Sprintf("预支付失败：%s", resp.ReturnMsg))
	}
	if resp.ReturnCode == "FAIL" {
		param := bm.JsonBody()
		logger.Error("func",
			zap.String("call", "WxPay_UnifiedOrder"),
			zap.String("params", param),
			zap.String("return_code", "FAIL"),
			zap.String("err_code", resp.ErrCode),
			zap.String("error", resp.ReturnMsg))
		return "",  "","",status.Error(10006, fmt.Sprintf("预支付失败：%s", resp.ReturnMsg))
	}
	if resp.ResultCode == "FAIL" {
		param := bm.JsonBody()
		logger.Error("func",
			zap.String("call", "WxPay_UnifiedOrder"),
			zap.String("params", param),
			zap.String("result_code", "FAIL"),
			zap.String("err_code", resp.ErrCode),
			zap.String("error", resp.ErrCodeDes))
		return "",  "", "",status.Error(10006, fmt.Sprintf("预支付失败：%s", resp.ErrCodeDes))
	}

	if resp.PrepayId == "" {
		param := bm.JsonBody()
		logger.Error("func",
			zap.String("call", "WxPay_UnifiedOrder"),
			zap.String("params", param),
			zap.String("result_code", "FAIL"),
			zap.String("err_code", resp.ErrCode),
			zap.String("error prepay id empty", resp.ErrCodeDes))
		return "",  "", "",status.Error(10006, fmt.Sprintf("预支付失败：%s", resp.ErrCodeDes))
	}


	now := time.Now()

	timeStamp := strconv.FormatInt(now.Unix(), 10)
	pac := "prepay_id=" + resp.PrepayId
	paySign := wechat.GetJsapiPaySign(info.AppId, resp.NonceStr,
		pac, wechat.SignType_MD5, timeStamp, info.ApiKey)
	wxPrepay := &WxPrepay{
		//PrepayID:  unifyOrderResp.PrepayId,
		AppID: info.AppId,
		//PartnerID: unifyOrderReq.Mch_id,
		Package:  pac,
		NonceStr:  resp.NonceStr,
		Timestamp: now.Unix(),
		SignType:  wechat.SignType_MD5,
		Sign:      paySign, // 回包给客户端需要二次签名
	}

	ret := ""
	if s, err := json.Marshal(wxPrepay); err != nil {
		return "",  "","",status.Error(10006, fmt.Sprintf("预支付失败：%s", err))
	} else {
		ret = string(s)
	}

	return ret, resp.PrepayId, resp.CodeUrl, nil

}





func WxAppletPrepay(ctx context.Context, req *pb_v1.WxAppletPrepayRequest) (reply *pb_v1.WxAppletPrepayReply, err error) {
	reply = &pb_v1.WxAppletPrepayReply{}

	// 捕获各个task中的异常并返回给调用者
	defer func() {
		if r := recover(); r != nil {
			err = fmt.Errorf("%+v", r)
			e := &status.Status{}
			if er := json.Unmarshal([]byte(err.Error()), e); er != nil {
				logger.Error("err",
					zap.String("system_err", err.Error()),
					zap.Stack("stacktrace"))
			}
		}
	}()
	if req.Order == "" || req.Product == "" || req.MchId == ""{
		return nil, errors.ParamsError
	}
	mreq := PrePayBaseInfo{
		AppId:parser.Conf.ThirdParty.Wx.AppletAppId,
		MchId:req.MchId,
		ApiKey:parser.Conf.ThirdParty.Wx.AppletAppKey,
		OpenId:req.OpenId,
		NotifyUrl:parser.Conf.ThirdParty.Wx.AppletNotifyUrl,
		ProductName:req.Product,
		ClientIp:req.InputIp,
		Amount:req.Amount,
		OrderNo:req.Order,
		Native:req.Native,
	}
	reply.Prepay, reply.PrepayId, reply.CodeUrl, err = prePay(&mreq)

	return reply, nil
}
