package wxapplet

import (
	"bytes"
	"context"
	"fmt"
	"git.getensh.com/common/gopkgs/logger"
	"github.com/tidwall/gjson"
	"go.uber.org/zap"
	"google.golang.org/grpc/status"
	"io/ioutil"
	"net/http"
	"property-thirdparty/errors"
	"property-thirdparty/parser"
	pb_v1 "property-thirdparty/pb/v1"
)




const WxPublicAddMiniUrl = "https://api.weixin.qq.com/cgi-bin/menu/create"
func WxPublicAddMini(ctx context.Context, req *pb_v1.WxPublicAddMiniRequest) (reply *pb_v1.WxPublicAddMiniReply, err error) {
	reply = &pb_v1.WxPublicAddMiniReply{}

	// 捕获各个task中的异常并返回给调用者
	defer func() {
		if r := recover(); r != nil {
			err = fmt.Errorf("%+v", r)
			e := &status.Status{}
			if er := json.Unmarshal([]byte(err.Error()), e); er != nil {
				logger.Error("err",
					zap.String("system_err", err.Error()),
					zap.Stack("stacktrace"))
			}
		}
	}()


	accessToken, err := GetPublicAccessToken()
	if err != nil {
		logger.Error("func",
			zap.String("call", "GetPublicAccessToken"),
			zap.String("error", err.Error()))
		return nil, errors.VendorError
	}

	subuton := map[string]string{
		"type": "miniprogram",
		"name": "小程序",
		"url":req.MiniUrl,
		"appid":parser.Conf.ThirdParty.Wx.AppletAppId,
		"pagepath":req.MiniPagepath,
	}
	subutons := []map[string]string{subuton}
	button := map[string]interface{}{
		"name":"小程序入口",
		"sub_button":subutons,
	}
	buttons := []map[string]interface{}{button}
	param := map[string]interface{}{
		"button":buttons,
	}
	bytesData, _ := json.Marshal(param)





	// 推送消息
	reader := bytes.NewReader(bytesData)
	url := fmt.Sprintf("%s?access_token=%s", WxPublicAddMiniUrl, accessToken)
	request, err := http.NewRequest("POST", url, reader)
	if err != nil {
		logger.Error("func",
			zap.String("call", "http.NewRequest"),
			zap.String("param", string(bytesData)),
			zap.String("error", err.Error()))
		return  nil, errors.VendorError
	}

	request.Header.Set("Content-Type", "application/json")


	client := http.Client{}
	resp, err := client.Do(request)
	if err != nil {
		logger.Error("func",
			zap.String("call", "http.client.Do"),
			zap.String("param", string(bytesData)),
			zap.String("error", err.Error()))
		return  nil, errors.VendorError
	}
	defer resp.Body.Close()

	// 处理消息结果
	respBytes, err := ioutil.ReadAll(resp.Body)
	if err != nil {
		logger.Error("func",
			zap.String("call", "ioutil.ReadAll"),
			zap.String("error", err.Error()))
		return  nil, errors.VendorError
	}


	errmsg := gjson.GetBytes(respBytes, "errmsg").String()
	errcode := gjson.GetBytes(respBytes, "errcode").Int()

	if errcode == 0 {
		return  reply, nil
	}

	logger.Error("func",
		zap.String("call", "WxPublicMsgSend"),
		zap.String("param", string(bytesData)),
		zap.String("error", fmt.Sprintf("%d %s", errcode, errmsg)))

	return  nil, errors.VendorError

}
func AddMiniTest() {
	mreq := pb_v1.WxPublicAddMiniRequest{}
	_, err := WxPublicAddMini(context.Background(), &mreq)
	fmt.Println(err)
}

